/*
 * Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana
 *
 * This file is part of blendb.
 *
 * blendb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * blendb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with blendb.  If not, see <http://www.gnu.org/licenses/>.
 */

import { expect } from "chai";

import { Hash } from "../util/hash";

import { Transformer } from "./transformer";
import { Source } from "./source";
import { Aggregate } from "./aggregate";

describe("transformer class", () => {
    const source = new Source("testSource");
    const aggregate = new Aggregate(["met:one"], ["dim:one", "dim:two"]);

    it("should be able to aggregate data", () => {
        source.truncate();

        for (let i = 0; i < 1000; i++) {
            source.push({
                id: i % 10,
                seed: Math.random()
            });
        }

        source.push({ id: 10, seed: 5 });
        source.push({ id: 10, seed: 5 });
        source.push({ id: 10, seed: 5 });
        source.push({ id: 10, seed: 5 });
        source.push({ id: 10, seed: 5 });

        const transformer = new Transformer("processTransformer", {
            source: source,
            destination: aggregate,
            functions: {
                map: ((doc: any, emit: Function) => {
                    emit({
                        "dim:one": doc.id,
                        "dim:two": Hash.sha1(doc.seed)
                    }, {
                        "met:one": Math.floor(doc.seed * 100000) / 100
                    });
                }),
                reduce: ((dimensions: any, metrics: any) => {
                    let tmp = 0;
                    metrics.forEach((met: any) => {
                        tmp += met["met:one"];
                    });
                    return {
                        "met:one": tmp
                    };
                })
            }
        });

        transformer.apply();

        let result = aggregate.find({
            "dim:one": 10,
            "dim:two": Hash.sha1(5)
        });

        expect(result).to.have.length(1);
        expect(result[0]).to.have.property("metrics");
        expect(result[0].metrics).to.have.property("met:one");
        expect(result[0].metrics["met:one"]).to.be.equal(25000);
    });
});
