/*
 * Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana
 *
 * This file is part of blendb.
 *
 * blendb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * blendb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with blendb.  If not, see <http://www.gnu.org/licenses/>.
 */

import { Hash } from "../util/hash";

import { Source } from "./source";
import { Aggregate } from "./aggregate";

export interface ITransformerOptions {
    source: Source;
    destination: Aggregate;
    functions: {
        map: (doc: any, emit: Function) => void;
        reduce: (dimensions: any, metrics: any) => any;
    };
}

export class Transformer {
    public name: string;
    public source: Source;
    public destination: Aggregate;
    private functions: any;

    constructor(name: string, options: ITransformerOptions) {
        this.name = name;
        this.source = options.source;
        this.destination = options.destination;
        this.functions = {
            map: options.functions.map,
            reduce: options.functions.reduce
        };
    }

    public apply() {
        let temp = new Map();

        this.destination.truncate();

        this.source.forEach((doc: any) => {
            let emit = (dimensions: any, metrics: any) => {
                let key = Hash.sha1(dimensions);
                let current = temp.get(key) || { dimensions, metrics: [] };
                temp.set(key, {
                    dimensions,
                    metrics: current.metrics.concat([metrics])
                });
            };

            this.functions.map(doc, emit);
        });

        temp.forEach((value, key) => {
            let dimensions = value.dimensions;
            let metrics = value.metrics;

            if (metrics.length > 1) {
                this.destination.push({
                    dimensions: dimensions,
                    metrics: this.functions.reduce(dimensions, metrics)
                });
            }
            else {
                this.destination.push({
                    dimensions: dimensions,
                    metrics: metrics
                });
            }
        });
    }
}
