/*
 * Copyright (C) 2016 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana
 *
 * This file is part of blend.
 *
 * blend is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * blend is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with blend.  If not, see <http://www.gnu.org/licenses/>.
 */

import { Dimension } from "./dimension";
import { Metric } from "./metric";
import { Hash } from "../util/hash";

export interface ViewOptions {
    metrics: Metric[];
    dimensions: Dimension[];
    materialized?: boolean;
    childViews?: View[];
}

export class View {
    public readonly id: string;
    public readonly metrics: Metric[];
    public readonly dimensions: Dimension[];
    public readonly materialized: boolean;
    public childViews: View[];

    constructor (options: ViewOptions) {
        this.metrics = options.metrics;
        this.dimensions = options.dimensions;
        this.materialized = options.materialized || true;
        this.childViews = options.childViews || [];

        // calculate the id of the view based on it's metrics and dimensions
        let metricsNames = this.metrics.map(metric => metric.name);
        let dimensionsNames = this.dimensions.map(dimension => dimension.name);
        this.id = Hash.sha1(metricsNames.concat(dimensionsNames));
    }
}
